﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Reflection;

namespace EntityDocumentation.Control
{
    /// <summary>
    /// Interaction logic for PropertyGrid.xaml
    /// </summary>
    public partial class PropertyGrid : UserControl
    {
        private object _itemsSource;
        List<TextBlock> _textBlock = new List<TextBlock>();
        List<TextBox> _textBox = new List<TextBox>();
        List<CheckBox> _checkBox = new List<CheckBox>();
        List<ComboBox> _comboBox = new List<ComboBox>();

        public PropertyGrid()
        {
            InitializeComponent();
        }

        public object ItemsSource
        {
            get
            {
                return _itemsSource;
            }

            set
            {
                _itemsSource = value;
                ReflectSource();
            }
        }

        private void ReflectSource()
        {
            int i = 0;
            int textBlockIndex = 0;
            int textBoxIndex = 0;
            int checkBoxIndex = 0;
            int comboBoxIndex = 0;

            foreach (TextBox ui in _textBox) BindingOperations.ClearAllBindings(ui);
            foreach (CheckBox ui in _checkBox) BindingOperations.ClearAllBindings(ui);
            foreach (ComboBox ui in _comboBox) BindingOperations.ClearAllBindings(ui);

            MainGrid.Children.Clear();

            if (_itemsSource == null) return;
            Type source = _itemsSource.GetType();

            SortedList<string, string> sortedProps = new SortedList<string, string>();
            foreach (PropertyInfo prop in source.GetProperties())
            {
                if (prop.PropertyType.IsGenericType || prop.PropertyType.IsClass && prop.PropertyType != typeof(string)) continue;
                if (MainGrid.Children.Count < i + 1)
                {
                    MainGrid.RowDefinitions.Add(new RowDefinition { Height = new GridLength(1, GridUnitType.Auto) });
                }
                sortedProps.Add(prop.Name, prop.Name);
                i += 1;
            }

            i = 0;
            foreach (string propName in sortedProps.Values)
            {
                PropertyInfo prop = source.GetProperty(propName);
                TextBlock name = GetTextBlock(ref textBlockIndex, prop.Name);
                UIElement value;
                if (prop.PropertyType.FullName == typeof(bool).FullName)
                {
                    value = GetCheckBox(ref checkBoxIndex, prop.Name);
                }
                else if (prop.PropertyType.IsEnum)
                {
                    value = GetComboBox(ref comboBoxIndex, prop);
                }
                else
                {
                    value = GetTextBox(ref textBoxIndex, prop.Name);
                }
                MainGrid.Children.Add(name);
                MainGrid.Children.Add(value);
                Grid.SetRow(name, i);
                Grid.SetColumn(name, 0);
                Grid.SetRow(value, i);
                Grid.SetColumn(value, 1);
                i += 1;
            }
        }

        private TextBlock GetTextBlock(ref int index, string propName)
        {
            TextBlock name;
            if (_textBlock.Count < index + 1)
            {
                name = new TextBlock { Text = propName };
                _textBlock.Add(name);
            }
            else
            {
                name = _textBlock[index];
                name.Text = propName;
            }
            index += 1;
            return name;
        }

        private TextBox GetTextBox(ref int index, string propName)
        {
            TextBox value;
            if (_textBox.Count < index + 1)
            {
                value = new TextBox();
                value.Style = (Style)FindResource("TextBox");
                _textBox.Add(value);
            }
            else
            {
                value = _textBox[index];
            }
            Binding b = new Binding(propName) { Source = _itemsSource, UpdateSourceTrigger = UpdateSourceTrigger.PropertyChanged };
            value.SetBinding(TextBox.TextProperty, b);
            index += 1;
            return value;
        }

        private CheckBox GetCheckBox(ref int index, string propName)
        {
            CheckBox value;
            if (_checkBox.Count < index + 1)
            {
                value = new CheckBox();
                _checkBox.Add(value);
            }
            else
            {
                value = _checkBox[index];
            }
            Binding b = new Binding(propName) { Source = _itemsSource, UpdateSourceTrigger = UpdateSourceTrigger.PropertyChanged };
            value.SetBinding(CheckBox.IsCheckedProperty, b);
            index += 1;
            return value;
        }

        private ComboBox GetComboBox(ref int index, PropertyInfo prop)
        {
            ComboBox value;
            if (_comboBox.Count < index + 1)
            {
                value = new ComboBox();
                _comboBox.Add(value);
            }
            else
            {
                value = _comboBox[index];
            }
            Binding b = new Binding(prop.Name) { Source = _itemsSource, UpdateSourceTrigger = UpdateSourceTrigger.PropertyChanged };
            value.ItemsSource = prop.PropertyType.GetEnumValues();
            value.SetBinding(ComboBox.SelectedValueProperty, b);
            index += 1;
            return value;
        }
    }
}
