﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.Serialization;

namespace EntityDocumentation.Data
{
    [DataContract]
    public class Property
    {
        [DataMember]
        public string Name { get; set; }
        [DataMember]
        public string NameKnownInRest { get; set; }
        [DataMember]
        public string AvailableSinceRelease { get; set; }
        [DataMember]
        public string IntroducedByTeam { get; set; }
        [DataMember]
        public string IntroducedForProject { get; set; }
        [DataMember]
        public string Remarks { get; set; }
        [DataMember]
        public string PropertyURL { get; set; }
    }
    public class PropertyList : System.Collections.ObjectModel.ObservableCollection<Property> { }

    [DataContract]
    public class Entity
    {
        [DataMember]
        public string Name { get; set; }
        public string NameKnownInRest { get { return Name.Replace(".", "_").Replace(" ", "_"); } }
        [DataMember]
        public string EntityProviderName { get; set; }
        [DataMember]
        public bool CanCreate { get; set; }
        [DataMember]
        public bool CanRetrieve { get; set; }
        [DataMember]
        public bool CanUpdate { get; set; }
        [DataMember]
        public bool CanDelete { get; set; }
        [DataMember]
        public bool CanSave { get; set; }
        [DataMember]
        public bool CanOpenNew { get; set; }
        [DataMember]
        public bool CanValidate { get; set; }
        [DataMember]
        public bool CanRestCreate { get; set; }
        [DataMember]
        public bool CanRestRetrieve { get; set; }
        [DataMember]
        public bool CanRestUpdate { get; set; }
        [DataMember]
        public bool CanRestDelete { get; set; }
        [DataMember]
        public bool IsMetadataGenerated { get; set; }
        public bool IsDynamicMetadata { get { return !string.IsNullOrEmpty(MetadataProviderName); } }
        [DataMember]
        public string MetadataProviderName { get; set; }
        [DataMember]
        public bool SupportRetrieveSet { get; set; }
        [DataMember]
        public bool IsBrowseEntity { get; set; }
        [DataMember]
        public string EntitiesProviderName { get; set; }
        [DataMember]
        public string FixedCriteria { get; set; }
        public bool HasFixedCriteria { get { return !string.IsNullOrEmpty(FixedCriteria); } }
        [DataMember]
        public bool SupportSorting { get; set; }

        [DataMember]
        public string AvailableSinceRelease { get; set; }
        [DataMember]
        public string IntroducedByTeam { get; set; }
        [DataMember]
        public string IntroducedForProject { get; set; }
        [DataMember]
        public string Remarks { get; set; }
        [DataMember]
        public string EntityURL { get; set; }

        [DataMember]
        public PropertyList PropertyList { get; set; }
    }
    public class EntityList : System.Collections.ObjectModel.ObservableCollection<Entity> { }

    [DataContract]
    public class EntityProvider
    {
        [DataMember]
        public string Name { get; set; }
        [DataMember]
        public string Description { get; set; }
        [DataMember]
        public string AvailableSinceRelease { get; set; }
        [DataMember]
        public string IntroducedByTeam { get; set; }
        [DataMember]
        public string IntroducedForProject { get; set; }
    }
    public class EntityProviderList : System.Collections.ObjectModel.ObservableCollection<EntityProvider> { }

    [DataContract]
    public class EntitiesProvider
    {
        [DataMember]
        public string Name { get; set; }
        [DataMember]
        public string Description { get; set; }
        [DataMember]
        public string AvailableSinceRelease { get; set; }
        [DataMember]
        public string IntroducedByTeam { get; set; }
        [DataMember]
        public string IntroducedForProject { get; set; }
    }
    public class EntitiesProviderList : System.Collections.ObjectModel.ObservableCollection<EntitiesProvider> { }

    [DataContract]
    public class Action
    {
        [DataMember]
        public string Name { get; set; }
        public string NameKnownInRest { get { return Name.Replace(".", "_").Replace(" ", "_"); } }
        [DataMember]
        public string ActionProviderName { get; set; }
        [DataMember]
        public bool CanVerify { get; set; }
        [DataMember]
        public bool CanProcess { get; set; }
        public bool HasInputProperty { get { return InputPropertyList != null && InputPropertyList.Count > 0; } }
        public bool HasOutputProperty { get { return OutputPropertyList != null && OutputPropertyList.Count > 0; } }
        [DataMember]
        public string AvailableSinceRelease { get; set; }
        [DataMember]
        public string IntroducedByTeam { get; set; }
        [DataMember]
        public string IntroducedForProject { get; set; }
        [DataMember]
        public string Remarks { get; set; }
        [DataMember]
        public PropertyList InputPropertyList { get; set; }
        [DataMember]
        public PropertyList OutputPropertyList { get; set; }
    }

    public class ActionList : System.Collections.ObjectModel.ObservableCollection<Action> { }

    [DataContract]
    public class ActionProvider
    {
        [DataMember]
        public string Name { get; set; }
        [DataMember]
        public string Description { get; set; }
        [DataMember]
        public string AvailableSinceRelease { get; set; }
        [DataMember]
        public string IntroducedByTeam { get; set; }
        [DataMember]
        public string IntroducedForProject { get; set; }
    }
    public class ActionProviderList : System.Collections.ObjectModel.ObservableCollection<ActionProvider> { }

    // It is best to add features incrementally for efficiency of list usage.
    public enum FeatureEnum { ImportFromServiceLocator = 0 }

    [DataContract]
    public class SolutionArea
    {
        /// <summary>
        /// Standard/Default constructor for serialization
        /// </summary>
        public SolutionArea() { }

        /// <summary>
        /// Constructor when instantiating new SolutionArea so that the latest version of features list can be registered to avoid the need to run patching code. Running patching code would mean running the same code more than once.
        /// </summary>
        /// <param name="newVersion"></param>
        public SolutionArea(bool newVersion)
        {
            currentVersion = new List<int>();
            SetLatestVersion();
            // Copy latest version
            foreach (var version in latestVersion) currentVersion.Add(version);
        }

        [DataMember]
        public string Name { get; set; }
        [DataMember]
        public string ServiceLocator { get; set; }
        [DataMember]
        public string Descripion { get; set; }
        [DataMember]
        public EntityList EntityList { get; set; }
        [DataMember]
        public ActionList ActionList { get; set; }

        #region Version
        /*
            Versioning is apply to each solution area. Versioning can be apply to different features and each of the feature can be at a different version.
            FeatureEnum consists of the list of features available for versioning.
            SavedVersion hold the saved features list version in a string by combining each of the feature version separated by a ".".
         */
        private List<int> currentVersion = null;
        private List<int> latestVersion = new List<int>();

        [DataMember]
        public string SavedVersion { get; set; } // Introduce version for patching purposes.

        public FeatureEnum Feature { get; set; }
        
        // Set the latest version of feature list.
        private void SetLatestVersion()
        {
            latestVersion = new List<int>();
            BuildFirstVersionList(latestVersion);
            latestVersion[(int)FeatureEnum.ImportFromServiceLocator] = 1;
        }

        // Build the first initial of version list.
        private void BuildFirstVersionList(List<int> version)
        {
            // Find out what is the max feature list available.
            var x = Enum.GetValues(typeof(FeatureEnum)).GetEnumerator();
            int maxFeature = 0;
            while (x.MoveNext())
                maxFeature = (int)x.Current;

            // Ensure new features are added to feature version list.
            for (int i = version.Count; i <= maxFeature; i++)
                version.Add(0);
        }

        // Set version based on SavedVersion (if exists)
        private void SetVersion()
        {
            currentVersion = new List<int>();
            if (latestVersion == null) SetLatestVersion();
            // convert version string to feature version list.
            if (!string.IsNullOrEmpty(SavedVersion))
                foreach (string v in SavedVersion.Split('.'))
                    currentVersion.Add(Convert.ToInt32(v));
            BuildFirstVersionList(currentVersion);
        }

        /// <summary>
        /// Build the version according to the feature list.
        /// </summary>
        public void BuildSaveVersion()
        {
            SavedVersion = string.Empty;
            if (currentVersion == null) SetVersion();
            foreach (var version in currentVersion)
                SavedVersion = string.Format("{0}.", version);
            if (!string.IsNullOrEmpty(SavedVersion)) SavedVersion = SavedVersion.Remove(SavedVersion.Length - 1);
        }

        /// <summary>
        /// Get the current version based on feature.
        /// </summary>
        /// <param name="feature">The feature where the version is to be retrieved</param>
        /// <returns>Returns the current version of the specified feature</returns>
        public int CurrentVersion(FeatureEnum feature)
        {
            if (currentVersion == null) SetVersion();
            return currentVersion[(int)feature];
        }

        /// <summary>
        /// Current version has been verified by calling methods. This can result in either the version of the feature increased if it is back on synch with the latest version or remain unchanged if it is already the correct version.
        /// </summary>
        /// <param name="feature">The feature that has been verified</param>
        public void CurrentVersionVerified(FeatureEnum feature)
        {
            int f = (int)feature;
            if (currentVersion == null) SetVersion();
            if (currentVersion[f] < latestVersion[f]) currentVersion[f] = latestVersion[f];
        }
        #endregion
    }
    public class SolutionAreaList : System.Collections.ObjectModel.ObservableCollection<SolutionArea> { }

    [DataContract]
    public class DocumentHeader
    {
        [DataMember]
        public string Product { get; set; }
        [DataMember]
        public SolutionAreaList SolutionArea { get; set; }
        [DataMember]
        public EntityProviderList EntityProvider { get; set; }
        [DataMember]
        public EntitiesProviderList EntitiesProvider { get; set; }
        [DataMember]
        public ActionProviderList ActionProvider { get; set; }
        public string Prefix { get; set; }
    }
}
