﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace EntityDocumentation.Data
{
    /// <summary>
    /// Progress bar features for Entity Documentation usage. Features include progress animation status and progress status message.
    /// </summary>
    class ProgressBar
    {
        private delegate void UpdateProgressBarDelegate(System.Windows.DependencyProperty dp, Object value);

        private System.Windows.Controls.ProgressBar progressBar;
        private System.Windows.Controls.TextBlock progressText;
        private UpdateProgressBarDelegate updatePbDelegate = null;

        /// <summary>
        /// Constructor of ProgressBar class
        /// </summary>
        /// <param name="progressBar">Defined ProgressBar component</param>
        /// <param name="progressText">Defined TextBlock meant for progress status message</param>
        public ProgressBar(System.Windows.Controls.ProgressBar progressBar, System.Windows.Controls.TextBlock progressText)
        {
            progressBar.Visibility = System.Windows.Visibility.Collapsed;
            progressText.Visibility = System.Windows.Visibility.Collapsed;
            this.progressBar = progressBar;
            this.progressText = progressText;
            updatePbDelegate = new UpdateProgressBarDelegate(progressBar.SetValue);
        }

        /// <summary>
        /// Start progress status by initializing progress bar animation and message status.
        /// Progress bar and text message will be visible.
        /// </summary>
        /// <param name="min">The lower bound of a progress bar animation</param>
        /// <param name="max">The upper bound of a progress bar anumation</param>
        /// <param name="status">The starting status message of a progress bar</param>
        public void Start(int min, int max, string status)
        {
            progressBar.Minimum = min;
            progressBar.Maximum = max;
            progressBar.Value = min;
            progressText.Text = status;
            progressBar.Background = System.Windows.Media.Brushes.Red;          // starting background.
            progressText.FontSize = 20;
            progressText.Foreground = System.Windows.Media.Brushes.WhiteSmoke;  // progress text.
            progressBar.Visibility = System.Windows.Visibility.Visible;         // make progress bar and text visible.
            progressText.Visibility = System.Windows.Visibility.Visible;
        }

        /// <summary>
        /// Stop a progress animation.
        /// Progress bar and text message will be hidden (collpased).
        /// </summary>
        public void Stop()
        {
            progressBar.Visibility = System.Windows.Visibility.Collapsed;
            progressText.Visibility = System.Windows.Visibility.Collapsed;
        }

        /// <summary>
        /// Animate progress bar and display the given status message.
        /// </summary>
        /// <param name="status"></param>
        public void Progress(string status)
        {
            progressBar.Value++;
            progressText.Text = string.Format("{0} {1}/{2}",status,progressBar.Value,progressBar.Maximum);
            System.Windows.Threading.Dispatcher.CurrentDispatcher.Invoke(updatePbDelegate, System.Windows.Threading.DispatcherPriority.Background, new object[] { System.Windows.Controls.ProgressBar.ValueProperty, progressBar.Value });
        }
    }
}
