﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.IO;
using System.Reflection;

namespace EntityDocumentation.Html
{
    /// <summary>
    /// Interaction logic for Entity.xaml
    /// </summary>
    public partial class Entity : Window
    {
        #region Html constant
        const string tableConstantStart = "<table border=\"1\" cellspacing=\"1\" style=\"border-collapse: collapse\">";
        const string headerTextFormat = "<td bgcolor=#C0504D><font color=white><b>{0}</b></td>{1}";
        const string alternateColorFormat = "<td bgColor=#EFD3D2>";
        const string centerAlignedFormat = "<td align=\"center\">";
        const string alternateColorCenterAlignedFormat = "<td bgColor=#EFD3D2 align=\"center\">";
        #endregion

        #region local variables
        #region Entity
        Dictionary<string, string> _entityColumn = new Dictionary<string, string>();
        Dictionary<string, string> _propertyColumn = new Dictionary<string, string>();
        Dictionary<string, string> _extendedPropertyColumn = new Dictionary<string, string>();
        Dictionary<string, string> _detailPropertyColumn = new Dictionary<string, string>();
        Dictionary<string, string> _entityProviderColumn = new Dictionary<string, string>();
        Dictionary<string, string> _entitiesProviderColumn = new Dictionary<string, string>();
        #endregion

        #region Action
        Dictionary<string, string> _actionColumn = new Dictionary<string, string>();
        Dictionary<string, string> _actionPropertyColumn = new Dictionary<string, string>();
        Dictionary<string, string> _actionExtendedPropertyColumn = new Dictionary<string, string>();
        Dictionary<string, string> _actionDetailPropertyColumn = new Dictionary<string, string>();
        #endregion

        string _product = string.Empty;
        bool viewOnly = false;
        private bool loaded = false;
        

        #region Progress Bar
        private Data.ProgressBar mainProgress = null;
        private Data.ProgressBar solutionProgress = null;
        private Data.ProgressBar detailsProgress = null;
        #endregion
        #endregion

        #region Constructor
        public Entity(bool viewOnly)
        {
            InitializeComponent();
            this.viewOnly = viewOnly;
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {

        }

        private void Window_Activated(object sender, EventArgs e)
        {
            if (loaded) return;
            loaded = true;

            InitializeProgressBar();

            SetEntityColumns();
            SetPropertyColumns();
            SetEntityProviderColumns();
            SetEntitiesProviderColumns();
            SetActionColumns();
            SetActionPropertyColumns();

            _product = Helper.ProductPrefix;

            if (string.IsNullOrEmpty(_product))
            {
                MessageBox.Show("Unrecognized product", this.Title, MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }

            //string file = string.Format("{0}\\{1}.Entity.html", Helper.myDomainPath, _product);
            string file = string.Format("{0}\\{1}.Main.html", Helper.DocumentationDomainPath, _product);
            if (!viewOnly)
            {
                mainProgress.Start(0, 8, "");
                File.WriteAllText(file, GenerateServceHtml());
                GenerateEntityProvidersOverviewHtml();
                GenerateEntitiesProvidersOverviewHtml();
                GeneratePropertiesDetailsOverviewHtml();
                GenerateEntitiesOverviewHtml();

                GenerateActionsOverviewHtml();
                GenerateActionPropertiesOverviewHtml();
                GenerateActionProvidersOverviewHtml();
            }
            mainProgress.Stop();
            brw.Navigate(new Uri(file));
        }
        #endregion

        private void InitializeProgressBar()
        {
            mainProgress = new Data.ProgressBar(progressBarMain, progressTextMain);
            solutionProgress = new Data.ProgressBar(progressBarSolution, progressTextSolution);
            detailsProgress = new Data.ProgressBar(progressBarDetails, progressTextDetails);
        }

        #region Set entity columns structure
        private void SetEntityColumns()
        {
            _entityColumn.Add("Name", "Name");
            _entityColumn.Add("NameKnownInRest", "Name (known in REST)");
            _entityColumn.Add("CanCreate", "Can create");
            _entityColumn.Add("CanRetrieve", "Can retrieve");
            _entityColumn.Add("CanUpdate", "Can update");
            _entityColumn.Add("CanDelete", "Can delete");
            _entityColumn.Add("CanSave", "Can save");
            _entityColumn.Add("CanOpenNew", "Can open new");
            _entityColumn.Add("CanValidate", "Can validate");
            _entityColumn.Add("CanRestCreate", "Can REST create");
            _entityColumn.Add("CanRestRetrieve", "Can REST retrieve");
            _entityColumn.Add("CanRestUpdate", "Can REST update");
            _entityColumn.Add("CanRestDelete", "Can REST delete");
            //_entityColumn.Add("IsMetadataGenerated", "Is metadata generated");
            //_entityColumn.Add("IsDynamicMetadata", "Is dynamic metadata");
            //_entityColumn.Add("MetadataProviderName", "Metadata provider name");
            _entityColumn.Add("SupportRetrieveSet", "Support Retrieve set");
            _entityColumn.Add("IsBrowseEntity", "Is popup browser");
            //_entityColumn.Add("EntitiesProviderName", "Entities provider name");
            //_entityColumn.Add("HasFixedCriteria", "Has fixed criteria");
            //_entityColumn.Add("SupportSorting", "Support sorting");
            _entityColumn.Add("AvailableSinceRelease", "Available since release");
            _entityColumn.Add("IntroducedByTeam", "Introduced by which team");
            _entityColumn.Add("IntroducedForProject", "Introduced for which project");
            _entityColumn.Add("Remarks", "Additional remarks");
            _entityColumn.Add("EntityProviderName", "Entity provider name");
            _entityColumn.Add("EntitiesProviderName", "Entities provider name");
        }

        private void SetPropertyColumns()
        {
            _propertyColumn.Add("Name", "Name");
            _propertyColumn.Add("Caption", "Caption");
            _propertyColumn.Add("Description", "Description");
            _propertyColumn.Add("Type", "Type");
            _propertyColumn.Add("Mandatory", "Mandatory");
            //_propertyColumn.Add("Length", "Length");
            //_propertyColumn.Add("Scale", "Scale");
            //_propertyColumn.Add("DefaultValue", "Default value");
            //_propertyColumn.Add("LowerRange", "Lower range");
            //_propertyColumn.Add("UpperRange", "Upper range");
            //_propertyColumn.Add("SelectionValues", "Selection values");
            //_propertyColumn.Add("AllowedCharacters", "Allowed characters");
            //_propertyColumn.Add("SourceName", "Source name");
            //_propertyColumn.Add("AutoIncrement", "Auto increment");
            //_propertyColumn.Add("NumericString", "Numeric string");
            //_propertyColumn.Add("RightAligned", "Right aligned");
            //_propertyColumn.Add("Unsigned", "Unsigned");
            //_propertyColumn.Add("UpperCase", "Upper case");
            //_propertyColumn.Add("ZeroEqualsNull", "Zero equals null");
            //_propertyColumn.Add("ZeroFill", "Zero fill");

            _detailPropertyColumn.Add("Name", "Name");
            _detailPropertyColumn.Add("SourceName", "Source name");
            _detailPropertyColumn.Add("Type", "Type");
            _detailPropertyColumn.Add("Mandatory", "Mandatory");
            _detailPropertyColumn.Add("Length", "Length");
            _detailPropertyColumn.Add("Scale", "Scale");
            _detailPropertyColumn.Add("DefaultValue", "Default value");
            _detailPropertyColumn.Add("LowerRange", "Lower range");
            _detailPropertyColumn.Add("UpperRange", "Upper range");
            _detailPropertyColumn.Add("AllowedCharacters", "Allowed characters");
            _detailPropertyColumn.Add("AutoIncrement", "Auto increment");
            _detailPropertyColumn.Add("NumericString", "Numeric string");
            _detailPropertyColumn.Add("RightAligned", "Right aligned");
            _detailPropertyColumn.Add("Unsigned", "Unsigned");
            _detailPropertyColumn.Add("UpperCase", "Upper case");
            _detailPropertyColumn.Add("ZeroEqualsNull", "Zero equals null");
            _detailPropertyColumn.Add("ZeroFill", "Zero fill");
            _detailPropertyColumn.Add("SelectionValues", "Selection values");

            _extendedPropertyColumn.Add("NameKnownInRest", "Name (known in REST)");
            _extendedPropertyColumn.Add("AvailableSinceRelease", "Available since release");
            _extendedPropertyColumn.Add("IntroducedByTeam", "Introduced by which team");
            _extendedPropertyColumn.Add("IntroducedForProject", "Introduced for which project");
            _extendedPropertyColumn.Add("Remarks", "Additional remarks");
        }

        private void SetEntityProviderColumns()
        {
            _entityProviderColumn.Add("Name", "Name");
            _entityProviderColumn.Add("Description", "Description");
            _entityProviderColumn.Add("AvailableSinceRelease", "Available since release");
            _entityProviderColumn.Add("IntroducedByTeam", "Introduced by which team");
            _entityProviderColumn.Add("IntroducedForProject", "Introduced for which project");
        }

        private void SetEntitiesProviderColumns()
        {
            _entitiesProviderColumn.Add("Name", "Name");
            _entitiesProviderColumn.Add("Description", "Description");
            _entitiesProviderColumn.Add("AvailableSinceRelease", "Available since release");
            _entitiesProviderColumn.Add("IntroducedByTeam", "Introduced by which team");
            _entitiesProviderColumn.Add("IntroducedForProject", "Introduced for which project");
        }
        #endregion

        #region Set action column structure
        private void SetActionPropertyColumns()
        {
            _actionPropertyColumn.Add("Name", "Name");
            _actionPropertyColumn.Add("Description", "Description");
            _actionPropertyColumn.Add("Type", "Type");
            _actionPropertyColumn.Add("Mandatory", "Mandatory");

            //_actionDetailPropertyColumn.Add("Name", "Name");
            //_actionDetailPropertyColumn.Add("Type", "Type");
            //_actionDetailPropertyColumn.Add("Mandatory", "Mandatory");
            _actionPropertyColumn.Add("Length", "Length");
            _actionPropertyColumn.Add("Scale", "Scale");
            _actionPropertyColumn.Add("SelectionValues", "Selection values");

            _actionExtendedPropertyColumn.Add("NameKnownInRest", "Name (known in REST)");
            _actionExtendedPropertyColumn.Add("AvailableSinceRelease", "Available since release");
            _actionExtendedPropertyColumn.Add("IntroducedByTeam", "Introduced by which team");
            _actionExtendedPropertyColumn.Add("IntroducedForProject", "Introduced for which project");
            _actionExtendedPropertyColumn.Add("Remarks", "Additional remarks");
        }

        private void SetActionColumns()
        {
            _actionColumn.Add("Name", "Name");
            _actionColumn.Add("NameKnownInRest", "Name (known in REST)");
            _actionColumn.Add("CanVerify", "Can verify");
            _actionColumn.Add("CanProcess", "Can process");
            _actionColumn.Add("HasInputProperty", "Has input property");
            _actionColumn.Add("HasOutputProperty", "Has output property");
            _actionColumn.Add("AvailableSinceRelease", "Available since release");
            _actionColumn.Add("IntroducedByTeam", "Introduced by which team");
            _actionColumn.Add("IntroducedForProject", "Introduced for which project");
            _actionColumn.Add("Remarks", "Additional remarks");
            _actionColumn.Add("ActionProviderName", "Action provider name");
        }
        #endregion

        /// <summary>
        /// Display the provider name. Trim left and replaced text with "..." if text is too long to be displayed.
        /// </summary>
        /// <param name="text">The text to be trim and replaced with "..."</param>
        /// <param name="length">The length of that triggers the trim process</param>
        /// <param name="start">The starting text location after the trim and replaced</param>
        /// <returns>Returns the left trim text with "..."</returns>
        private string TrimLeftText(string text, int length, int start)
        {
            if (text.Length > length) text = "... " + text.Substring(start);
            return text;
        }

        /// <summary>
        ///  Generate type of service html. (The starting html page)
        /// </summary>
        /// <returns>Returns the starting html page.</returns>
        private string GenerateServceHtml()
        {
            mainProgress.Progress("Generating type of services...");
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<html>");
            sb.AppendLine("<body>");

            sb.AppendFormat("<h1>List of Services for Release {0}</h1>",Helper.ProductRelease);
            sb.Append("<ul>");
            sb.AppendLine(string.Format("<li><a href=\"{0}\\{1}.Entity.html\">Entity</a>", Helper.DocumentationDomainPath, _product)); // Navigate to Entity html start page.
            sb.AppendLine(string.Format("<li><a href=\"{0}\\{1}.Action.html\">Action</a>", Helper.DocumentationDomainPath, _product)); // NAvigate to Action html start page.
            sb.Append("</ul>");

            sb.AppendLine("</body>");
            sb.AppendLine("</html>");

            return sb.ToString();
        }
        
        /// <summary>
        /// Generate remarks page for Entity and Action.
        /// </summary>
        /// <param name="service">The entity or action name</param>
        /// <param name="remarks">The remarks for the service</param>
        /// <param name="filePart">The file part of the remarks html. The filePart will be part of the html file name.</param>
        private void GenerateServiceRemarks(string service, string remarks, string filePart = ".Remarks")
        {
            StringBuilder sbError = new StringBuilder();
            try
            {
                if (!string.IsNullOrEmpty(remarks))
                {
                    StringBuilder sb = new StringBuilder();
                    sb.AppendLine("<html>");
                    sb.AppendLine("<body>");

                    sb.AppendLine(string.Format("<h1>Additional remarks for {0}</h1>", service));
                    foreach (string s in remarks.Split(new string[] { "\r\n", "\r", "\n" }, StringSplitOptions.None))
                        sb.AppendLine(string.Format("{0}</br>", s));    // Break line break with </br>

                    sb.AppendLine("</body>");
                    sb.AppendLine("</html>");

                    string file = string.Format("{0}{1}{2}.{3}.html", Helper.DocumentationHtmlPath, _product, filePart, service);
                    File.WriteAllText(file, sb.ToString());
                }
            }
            catch (Exception)
            {

            }
        }

        /// <summary>
        ///  Generate remarks page for services properties.
        /// </summary>
        /// <param name="service">The entity or action name</param>
        /// <param name="property">The property of specific service</param>
        /// <param name="remarks"></param>
        /// <param name="filePart">The file part of the remarks html. The filePart will be part of the html file name.</param>
        private void GeneratePropertyRemarks(string service, string property, string remarks, string filePart = ".Remarks")
        {
            StringBuilder sbError = new StringBuilder();
            try
            {
                StringBuilder sb = new StringBuilder();
                sb.AppendLine("<html>");
                sb.AppendLine("<body>");

                sb.AppendLine(string.Format("<h1>Additional remarks for {0}.{1}</h1>", service, property));
                foreach (string s in remarks.Split(new string[] { "\r\n", "\r", "\n" }, StringSplitOptions.None))
                    sb.AppendLine(string.Format("{0}</br>", s)); // Break line break with </br>

                sb.AppendLine("</body>");
                sb.AppendLine("</html>");

                string file = string.Format("{0}{1}{2}.{3}.{4}.html", Helper.DocumentationHtmlPath, _product, filePart, service, property);
                File.WriteAllText(file, sb.ToString());
            }
            catch (Exception)
            {

            }
        }

        #region Generate entity"
        
        /// Generate entity starting page. Generate list of entity grouped by Solution area and Entity.
        private void GenerateEntitiesOverviewHtml()
        {
            mainProgress.Progress("Generating Entities...");
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<html>");
            sb.AppendLine("<body>");
            sb.AppendFormat("<h1>List of Entities for Release {0}{1}</h1>",Helper.ProductRelease,Environment.NewLine);

            solutionProgress.Start(0, Helper.Header.SolutionArea.Count, "");

            //foreach (var solutionArea in from sa in Helper.Header.SolutionArea orderby sa.Name select sa)
            foreach (var solutionArea in Helper.Header.SolutionArea.OrderBy(sa=>sa.Name))            
            { // List of solution areas.
                solutionProgress.Progress(string.Format("Solution: {0}", solutionArea.Name));
                sb.AppendLine("<h2>Solution area</h2>");
                sb.AppendFormat("<h3>{0}</h3>{1}", solutionArea.Name, Environment.NewLine);

                GenerateEntitiesOverviewHtml(sb, solutionArea);                
            }
            detailsProgress.Stop();
            solutionProgress.Stop();
            sb.AppendLine("</body>");
            sb.AppendLine("</html>");

            string.Format("{0}\\{1}.Entity.html", Helper.DocumentationDomainPath, _product);
            File.WriteAllText(string.Format("{0}\\{1}.Entity.html", Helper.DocumentationDomainPath, _product), sb.ToString());
        }

        // Generate html for Entities of specific solution area.
        private void GenerateEntitiesOverviewHtml(StringBuilder sb, Data.SolutionArea solutionArea)
        {
            sb.AppendLine(tableConstantStart);

            bool alternateColor = false;
            bool isHeader = true;
            char previousGroup = '!';
            //var entities = from l in solutionArea.EntityList orderby l.Name select l;
            var entities = solutionArea.EntityList.OrderBy(l => l.Name);
            detailsProgress.Start(0, entities.Count(), "");
            foreach (var entity in entities)
            { // List of entities
                detailsProgress.Progress(string.Format("Entity: {0}", entity.Name));
                string bgColor = alternateColor ? "<td>" : alternateColorFormat;
                string bgColorCenter = alternateColor ? centerAlignedFormat : alternateColorCenterAlignedFormat;
                Type entityType = entity.GetType();
                if (previousGroup != (char)entity.Name[0])
                {
                    previousGroup = entity.Name[0];
                    sb.AppendLine("<tr>");
                    sb.AppendLine(string.Format("<td colspan=\"{0}\"><i></br>Entity starting with {1}</i></td>", _entityColumn.Count, previousGroup));
                    sb.AppendLine("</tr>");
                    isHeader = true;
                }

                if (isHeader)
                {
                    sb.AppendLine("<tr>");
                    foreach (var prop in _entityColumn)
                        sb.AppendFormat(headerTextFormat, prop.Value, Environment.NewLine);
                    sb.AppendLine("</tr>");
                }
                sb.AppendLine("<tr>");

                GenerateEntitiesOverviewPropertiesOverviewHtml(entityType, entity, sb, bgColorCenter, bgColor);

                sb.AppendLine("</tr>");
                alternateColor = !alternateColor;
                isHeader = false;
            }
            sb.AppendLine("</table>");
        }

        // Generate html of entity with its properties.        
        private void GenerateEntitiesOverviewPropertiesOverviewHtml(Type entityType, Data.Entity entity, StringBuilder sb, string bgColorCenter, string bgColor)
        {
            // entity properties
            foreach (var entityColumn in _entityColumn)
            {
                var prop = entityType.GetProperty(entityColumn.Key);

                object value = prop.GetValue(entity, null);
                if (value == null) value = "";
                if (value is bool)
                    sb.AppendLine(bgColorCenter);
                else
                    sb.AppendLine(bgColor);

                switch (prop.Name)
                {
                    case "Name":
                        sb.AppendLine(string.Format("<a href=\"html\\{0}.Entity.{1}.html\">{2}</a>", _product, entity.Name, value.ToString()));
                        break;
                    case "EntityProviderName":
                        sb.AppendLine(string.Format("<a href=\"html\\{0}.EntityProvider.{1}.html\" title=\"{1}\">{2}</a>", _product, value.ToString(), TrimLeftText(value.ToString(), 24, 22)));
                        break;
                    case "EntitiesProviderName":
                        sb.AppendLine(string.Format("<a href=\"html\\{0}.EntitiesProvider.{1}.html\" title=\"{1}\">{2}</a>", _product, value.ToString(), TrimLeftText(value.ToString(), 26, 24)));
                        break;
                    case "Remarks":
                        string remarks = (string)value;
                        GenerateServiceRemarks(entity.Name, remarks);
                        if (remarks.Length > 20) value = remarks.Substring(0, 20) + "...";
                        sb.AppendLine(string.Format("<a href=\"html\\{0}.Remarks.{1}.html\" title=\"{2}\">{3}</a>", _product, entity.Name, remarks, value.ToString()));
                        break;
                    default:
                        if (value is bool)
                        {
                            if ((bool)value) sb.AppendLine("<img src=\"image\\check.png\"/>");
                        }
                        else
                            sb.AppendLine(value.ToString());
                        break;
                }
                sb.AppendLine("</td>");
            }
        }

        // Generate properties details heading properties (Name, description, type, etc)
        private void GeneratePropertiesDetailsOverviewPropertyDetails(Exact.Services.MetaModel.Entity.Data.MetadataProperty p, Type propType, StringBuilder sb, string bgColorCenter, string bgColor)
        {
            bool firstColumn = true;
            foreach (var propColumn in _propertyColumn)             //property details
            {
                var prop = propType.GetProperty(propColumn.Key);    //Get info from metadata.

                object value = prop.GetValue(p, null);
                if (value == null) value = "";
                if (value is bool)
                    sb.AppendLine(bgColorCenter);
                else
                    sb.AppendLine(bgColor);

                if (firstColumn)
                { // First column requires hyperlink.
                    sb.AppendLine(string.Format("<a href=\"#{0}\">{0}</a>", value.ToString()));
                    firstColumn = false;
                }
                else
                {
                    if (value is bool)
                    { // Display check box for boolean column.
                        if ((bool)value) sb.AppendLine("<img src=\"..\\image\\check.png\"/>");
                    }
                    else
                        sb.AppendLine(value.ToString());
                }
                sb.AppendLine("</td>");
            }
        }

        // Generate extended properties (Introduced in which release, by which team, for which project, etc)
        private void GeneratePropertiesDetailOverviewPropertyExtendedDetails(StringBuilder sb, string propertyName, Data.Entity entity, string bgColorCenter, string bgColor)
        {
            Type extendedPropType = typeof(Data.Property);
            var propValue = entity.PropertyList.FirstOrDefault(pl => pl.Name == propertyName);
            foreach (var propColumn in _extendedPropertyColumn)             //extended property details
            {
                var prop = extendedPropType.GetProperty(propColumn.Key);
                //object value = prop.GetValue(entity.PropertyList[i], null);                                
                object value = prop.GetValue(propValue, null);
                if (value == null) value = "";
                if (value is bool)
                    sb.AppendLine(bgColorCenter);
                else
                    sb.AppendLine(bgColor);

                if (propColumn.Key == "Remarks")
                {
                    string remarks = (string)value;
                    if (!string.IsNullOrEmpty(remarks))
                    {
                        GeneratePropertyRemarks(entity.Name, propertyName, remarks);
                        if (remarks.Length > 20) value = remarks.Substring(0, 20) + "...";
                        sb.AppendLine(string.Format("<a href=\"{0}.Remarks.{1}.{2}.html\" title=\"{3}\">{4}</a>", _product, entity.Name, propertyName, remarks, value.ToString()));
                    }
                }
                else
                {
                    if (value is bool)
                    {
                        if ((bool)value) sb.AppendLine("<img src=\"..\\image\\check.png\"/>");
                    }
                    else
                        sb.AppendLine(value.ToString());
                }
                sb.AppendLine("</td>");
            }
        }

        // Generate the properties attributes in a different table. (Mandatory, Upper case, Selection values,  etc)
        private void GeneratePropertiesDetailOverviewPropertyAttributes(StringBuilder sb, Exact.Services.MetaModel.Entity.Data.MetadataProperty p, Type propType, string bgColorCenter, string bgColor)
        {
            bool firstColumn = true;
            foreach (var propColumn in _detailPropertyColumn)             // details property details
            {
                var prop = propType.GetProperty(propColumn.Key);
                object value = prop.GetValue(p, null);
                if (value == null) value = "";

                if (value is bool)
                    sb.AppendLine(bgColorCenter);
                else
                    sb.AppendLine(bgColor);

                if (propColumn.Key == "SelectionValues")
                {
                    if (!string.IsNullOrWhiteSpace((string)value))
                    {
                        string[] sv = ((string)value).Split(';');
                        value = tableConstantStart;
                        foreach (string v in sv)
                            value += string.Format("<tr><td>{0}</td></tr>", v);
                        value += "</table>";
                    }
                }
                if (firstColumn)
                {
                    sb.AppendLine(string.Format("<a name={0}>{0}</a>", value.ToString()));
                    firstColumn = false;
                }
                else
                {
                    if (value is bool)
                    {
                        if ((bool)value) sb.AppendLine("<img src=\"..\\image\\check.png\"/>");
                    }
                    else
                        sb.AppendLine(value.ToString());
                }
                sb.AppendLine("</td>");
            }
        }

        // Generate the list of property for specific entity of a solution area.
        private void GeneratePropertiesDetailsOverviewEntity(StringBuilder sb, StringBuilder sb2,Exact.Services.MetaModel.Entity.Data.MetadataEntity metadata, Data.Entity entity)
        {
            sb.AppendLine(tableConstantStart);
            sb2.AppendLine("</br><b>Detail information of properties</b>");
            sb2.AppendLine(tableConstantStart);

            int i = 0;
            bool alternateColor = false;
            bool isHeader = true;
            char previousGroup = '!';
            bool useGrouping = metadata.Interface.Properties.Count > 11;
            //foreach (var p in from mp in x.Interface.Properties orderby mp.Name select mp)
            foreach (var p in metadata.Interface.Properties.OrderBy(mp => mp.Name))
            {
                if (useGrouping && previousGroup != (char)p.Name[0])
                { // Grouping Properties.
                    previousGroup = p.Name[0];

                    sb.AppendLine("<tr>");
                    sb.AppendLine(string.Format("<td colspan=\"{0}\"><i></br>Property starting with {1}<i></td>", _propertyColumn.Count, previousGroup));
                    sb.AppendLine("</tr>");

                    sb2.AppendLine("<tr>");
                    sb2.AppendLine(string.Format("<td colspan=\"{0}\"><i></br>Property starting with {1}<i></td>", _propertyColumn.Count, previousGroup));
                    sb2.AppendLine("</tr>");

                    isHeader = true;
                }

                if (isHeader)
                { // Property header
                    sb.AppendLine("<tr>");
                    sb2.AppendLine("<tr>");
                    foreach (var prop in _propertyColumn) sb.AppendFormat(headerTextFormat, prop.Value, Environment.NewLine);           // property header
                    foreach (var prop in _extendedPropertyColumn) sb.AppendFormat(headerTextFormat, prop.Value, Environment.NewLine);   //extended property header
                    foreach (var prop in _detailPropertyColumn) sb2.AppendFormat(headerTextFormat, prop.Value, Environment.NewLine);    //detail property header
                    sb.AppendLine("</tr>");
                    sb2.AppendLine("</tr>");
                }
                isHeader = false;

                string bgColor = alternateColor ? "<td>" : alternateColorFormat;
                string bgColorCenter = alternateColor ? centerAlignedFormat : alternateColorCenterAlignedFormat;
                sb.AppendLine("<tr>");
                sb2.AppendLine("<tr>");
                Type propType = p.GetType();
                GeneratePropertiesDetailsOverviewPropertyDetails(p, propType, sb, bgColorCenter, bgColor);
                GeneratePropertiesDetailOverviewPropertyExtendedDetails(sb, p.Name, entity, bgColorCenter, bgColor);
                GeneratePropertiesDetailOverviewPropertyAttributes(sb2, p, propType, bgColorCenter, bgColor);
                sb.AppendLine("</tr>");
                sb2.AppendLine("</tr>");
                i++;
                alternateColor = !alternateColor;
            }

            sb.AppendLine("</table>");
            sb2.AppendLine("</table>");
        }

        private void GenerateObsoletePropertiesDetailsOverviewEntity(StringBuilder sb, Exact.Services.MetaModel.Entity.Data.MetadataEntity metadata, Data.Entity entity)
        {
            sb.AppendLine("</br><b>List of obsolete properties</b>");
            var obsoleteProperties = entity.PropertyList.Where(pl => !(metadata.Interface.Properties.Select(p => p.Name).Contains(pl.Name))).OrderBy(pp => pp.Name);
            if (obsoleteProperties.Count() == 0)
            {
                sb.AppendLine("<br>There is no obsolete property");
                return;
            }
            sb.AppendLine(tableConstantStart);

            int i = 0;
            bool alternateColor = false;

            // Property header
            sb.AppendLine("<tr>");
            sb.AppendFormat(headerTextFormat, "Name", Environment.NewLine);
            foreach (var prop in _extendedPropertyColumn) sb.AppendFormat(headerTextFormat, prop.Value, Environment.NewLine);   //extended property header
            sb.AppendLine("</tr>");

            foreach (var p in obsoleteProperties)
            {
                string bgColor = alternateColor ? "<td>" : alternateColorFormat;
                string bgColorCenter = alternateColor ? centerAlignedFormat : alternateColorCenterAlignedFormat;
                sb.AppendLine("<tr>");
                Type propType = p.GetType();
                sb.Append(bgColor).Append(p.Name).Append("</td>");
                GeneratePropertiesDetailOverviewPropertyExtendedDetails(sb, p.Name, entity, bgColorCenter, bgColor);
                sb.AppendLine("</tr>");
                i++;
                alternateColor = !alternateColor;
            }

            sb.AppendLine("</table>");
        }

        // Generate key property table of an entity in List of Properties page.
        private void GeneratePropertyKey(StringBuilder sb, Data.Entity entity,Exact.Services.MetaModel.Entity.Data.MetadataEntity metadata)
        {
            sb.AppendLine(tableConstantStart);
            sb.AppendLine("<tr>");
            sb.AppendFormat(headerTextFormat, "Entity", Environment.NewLine);
            sb.AppendFormat(headerTextFormat, entity.Name, Environment.NewLine);
            sb.AppendLine("</tr>");
            sb.AppendLine("<tr>");
            sb.AppendFormat(headerTextFormat, "Key property", Environment.NewLine);
            sb.AppendLine(string.Format("<td>{0}</td>", metadata.Interface.KeyProperty));
            sb.AppendLine("</tr>");
            sb.AppendLine("</table></br>");
        }

        // Generating the property details of each entities in each solution area.
        private void GeneratePropertiesDetailsOverviewHtml()
        {
            mainProgress.Progress("Generating entity's properties...");
            StringBuilder sbError = new StringBuilder();
            Exact.Services.MetaModel.Entity.Metadata meta = new Exact.Services.MetaModel.Entity.Metadata();
            solutionProgress.Start(0,Helper.Header.SolutionArea.Count,"");
            foreach (var solutionArea in Helper.Header.SolutionArea)
            {   // Build Solution area
                solutionProgress.Progress(string.Format("Solution: {0}", solutionArea.Name));
                detailsProgress.Start(0, solutionArea.EntityList.Count, "");
                foreach (var entity in solutionArea.EntityList)
                { // Entity                    
                    detailsProgress.Progress(string.Format("Entity: {0}", entity.Name));
                    try
                    {
                        StringBuilder sb = new StringBuilder();
                        StringBuilder sb2 = new StringBuilder();
                        sb.AppendLine("<html>");
                        sb.AppendLine("<body>");
                        sb.AppendLine("<h1>List of properties</h1>");

                        Exact.Services.MetaModel.Entity.Data.MetadataEntity x = meta.Retrieve(entity.Name);
                        GeneratePropertyKey(sb, entity, x);
                        GeneratePropertiesDetailsOverviewEntity(sb, sb2, x, entity);
                        sb.Append(sb2.ToString());
                        GenerateObsoletePropertiesDetailsOverviewEntity(sb, x, entity);                        

                        sb.AppendLine("</body>");
                        sb.AppendLine("</html>");
                        string file = string.Format("{0}{1}.Entity.{2}.html", Helper.DocumentationHtmlPath, _product, entity.Name);
                        File.WriteAllText(file, sb.ToString());
                    }
                    catch (Exception ex)
                    {
                        sbError.AppendLine(string.Format("{0}:{1}", solutionArea.Name, entity.Name));
                        if (ex.InnerException != null) sbError.AppendLine(string.Format(" - {0}", ex.InnerException.Message));
                    }
                }
            }
            if (sbError.Length > 0) MessageBox.Show(sbError.ToString(), this.Title, MessageBoxButton.OK, MessageBoxImage.Warning);
        }

        private void GenerateEntityProvidersOverviewHtml()
        {
            mainProgress.Progress("Generating entity providers...");
            Type providerType = typeof(Data.EntityProvider);
            //foreach (var ep in from e in Helper.Header.EntityProvider orderby e.Name select e)
            foreach (var ep in Helper.Header.EntityProvider.OrderBy(e => e.Name))
            {
                StringBuilder sb = new StringBuilder();
                sb.AppendLine("<html>");
                sb.AppendLine("<body>");
                sb.AppendLine("<h1>Entity provider details</h1>");

                sb.AppendLine(tableConstantStart);
                foreach (var epc in _entityProviderColumn)
                {
                    sb.AppendLine("<tr>");
                    sb.AppendFormat(headerTextFormat, epc.Value, Environment.NewLine);
                    sb.AppendLine(string.Format("<td>{0}</td>", providerType.GetProperty(epc.Key).GetValue(ep, null)));
                    sb.AppendLine("</tr>");
                }
                sb.AppendLine("</table>");

                sb.AppendLine("</body>");
                sb.AppendLine("</html>");

                string file = string.Format("{0}{1}.EntityProvider.{2}.html", Helper.DocumentationHtmlPath, _product, ep.Name);
                File.WriteAllText(file, sb.ToString());
            }
        }

        private void GenerateEntitiesProvidersOverviewHtml()
        {
            mainProgress.Progress("Generating entities providers...");
            Type providerType = typeof(Data.EntitiesProvider);
            foreach (var ep in from e in Helper.Header.EntitiesProvider orderby e.Name select e)
            {
                StringBuilder sb = new StringBuilder();
                sb.AppendLine("<html>");
                sb.AppendLine("<body>");
                sb.AppendLine("<h1>Retrieve Set provider details</h1>");

                sb.AppendLine(tableConstantStart);
                foreach (var epc in _entitiesProviderColumn)
                {
                    sb.AppendLine("<tr>");
                    sb.AppendFormat(headerTextFormat, epc.Value, Environment.NewLine);
                    sb.AppendLine(string.Format("<td>{0}</td>", providerType.GetProperty(epc.Key).GetValue(ep, null)));
                    sb.AppendLine("</tr>");
                }
                sb.AppendLine("</table>");

                sb.AppendLine("</body>");
                sb.AppendLine("</html>");

                string file = string.Format("{0}{1}.EntitiesProvider.{2}.html", Helper.DocumentationHtmlPath, _product, ep.Name);
                File.WriteAllText(file, sb.ToString());
            }
        }
        
        #endregion

        #region Generate action
        // Generate action overview main page.
        private void GenerateActionsOverviewHtml()
        {
            mainProgress.Progress("Generating Action...");
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<html>");
            sb.AppendLine("<body>");
            sb.AppendFormat("<h1>List of Actions for Release {0}{1}</h1>", Helper.ProductRelease,Environment.NewLine);

            solutionProgress.Start(0, Helper.Header.SolutionArea.Count, "");
            //foreach (var solutionArea in from sa in Helper.Header.SolutionArea orderby sa.Name select sa)
            foreach (var solutionArea in Helper.Header.SolutionArea.OrderBy(sa=>sa.Name))
            {
                solutionProgress.Progress(string.Format("Solution: {0}", solutionArea.Name));
                sb.AppendLine("<h2>Solution area</h2>");
                sb.AppendFormat("<h3>{0}</h3>{1}", solutionArea.Name, Environment.NewLine);

                GenerateActionOverviewHtml(sb, solutionArea);
            }

            sb.AppendLine("</body>");
            sb.AppendLine("</html>");

            File.WriteAllText(string.Format("{0}\\{1}.Action.html", Helper.DocumentationDomainPath, _product), sb.ToString());
            detailsProgress.Stop();
            solutionProgress.Stop();
        }

        // Generate entity and properties per solution area.
        private void GenerateActionOverviewHtml(StringBuilder sb, Data.SolutionArea solutionArea)
        {
            sb.AppendLine(tableConstantStart);

            if (solutionArea.ActionList == null)
            {
                sb.AppendFormat("<p>There is no action defined for this solution area.</p>");
            }
            else
            {
                bool alternateColor = false;
                bool isHeader = true;
                char previousGroup = '!';

                //var actions = from l in solutionArea.ActionList orderby l.Name select l;
                var actions = solutionArea.ActionList.OrderBy(l => l.Name);
                detailsProgress.Start(0, actions.Count(), "");
                foreach (var action in actions)
                {
                    detailsProgress.Progress(string.Format("Action: {0}", action.Name));
                    string bgColor = alternateColor ? "<td>" : alternateColorFormat;
                    string bgColorCenter = alternateColor ? centerAlignedFormat : alternateColorCenterAlignedFormat;
                    Type actionType = action.GetType();
                    if (previousGroup != (char)action.Name[0])
                    { // Grouping
                        previousGroup = action.Name[0];
                        sb.AppendLine("<tr>");
                        sb.AppendLine(string.Format("<td colspan=\"{0}\"><i></br>Entity starting with {1}</i></td>", _actionColumn.Count, previousGroup));
                        sb.AppendLine("</tr>");
                        isHeader = true;
                    }

                    if (isHeader)
                    { // Generate header
                        sb.AppendLine("<tr>");
                        foreach (var prop in _actionColumn)
                            sb.AppendFormat(headerTextFormat, prop.Value, Environment.NewLine);
                        sb.AppendLine("</tr>");
                    }
                    sb.AppendLine("<tr>");

                    GenerateActionOverviewPropertiesDetails(actionType, action, sb, bgColorCenter, bgColor);

                    sb.AppendLine("</tr>");
                    alternateColor = !alternateColor;
                    isHeader = false;
                }
            }
            sb.AppendLine("</table>");
        }

        // Generate List of action and its properties  (Name, Description, Type, etc)
        private void GenerateActionOverviewPropertiesDetails(Type actionType, Data.Action action, StringBuilder sb, string bgColorCenter, string bgColor)
        {
            // action properties
            foreach (var actionColumn in _actionColumn)
            {
                var prop = actionType.GetProperty(actionColumn.Key);
                object value = prop.GetValue(action, null);
                if (value == null) value = "";

                if (value is bool)
                    sb.AppendLine(bgColorCenter);
                else
                    sb.AppendLine(bgColor);

                switch (prop.Name)
                {
                    case "Name":
                        sb.AppendLine(string.Format("<a href=\"html\\{0}.Action.{1}.html\">{2}</a>", _product, action.Name, value.ToString()));
                        break;
                    case "ActionProviderName":
                        sb.AppendLine(string.Format("<a href=\"html\\{0}.ActionProvider.{1}.html\" title=\"{1}\">{2}</a>", _product, value.ToString(), TrimLeftText(value.ToString(), 24, 22)));
                        break;
                    case "Remarks":
                        string remarks = (string)value;
                        GenerateServiceRemarks(action.Name, remarks, ".Action.Remarks");
                        if (remarks.Length > 20) value = remarks.Substring(0, 20) + "...";
                        sb.AppendLine(string.Format("<a href=\"html\\{0}.Action.Remarks.{1}.html\" title=\"{2}\">{3}</a>", _product, action.Name, remarks, value.ToString()));
                        break;
                    default:
                        if (value is bool)
                        { // Display checkmark if value is boolean (true).
                            if ((bool)value) sb.AppendLine("<img src=\"image\\check.png\"/>");
                        }
                        else
                            sb.AppendLine(value.ToString());
                        break;
                }
                sb.AppendLine("</td>");
            }
        }

        // Generate list of Action properties (input and output)
        private void GenerateActionPropertiesOverviewHtml(StringBuilder sb, bool isInput, Exact.Services.MetaModel.Action.Metadata meta, Data.Action action)
        {
            string title = string.Empty;
            Exact.Services.MetaModel.Action.Data.MetadataAction metadata = null;
            string inputOrOutput = string.Empty;

            // Setting up input and output configuration.
            if (isInput)
            {
                metadata = meta.RetrieveInputMetadata(action.Name);
                inputOrOutput = "input";
            }
            else
            {
                metadata = meta.RetrieveOutputMetadata(action.Name);
                inputOrOutput = "output";
            }

            sb.AppendFormat("<h2>List of {0} property</h2>{1}", inputOrOutput, Environment.NewLine);
            if (metadata.Properties.Count == 0) sb.AppendFormat("<p>There is no {0} property defined for {1} action</p>{2}", inputOrOutput, action.Name, Environment.NewLine);

            sb.AppendLine(tableConstantStart);

            int i = 0;
            bool alternateColor = false;
            bool isHeader = true;
            bool firstColumn = true;
            foreach (var p in metadata.Properties.OrderBy(mp=> mp.Name))
            {
                if (isHeader)
                { // Generate header
                    sb.AppendLine("<tr>");
                    foreach (var prop in _actionPropertyColumn)         // property header
                        sb.AppendFormat(headerTextFormat, prop.Value, Environment.NewLine);
                    foreach (var prop in _actionExtendedPropertyColumn) //extended property header
                        sb.AppendFormat(headerTextFormat, prop.Value, Environment.NewLine);
                    sb.AppendLine("</tr>");
                }

                string bgColor = alternateColor ? "<td>" : alternateColorFormat;
                string bgColorCenter = alternateColor ? centerAlignedFormat : alternateColorCenterAlignedFormat;
                isHeader = false;
                sb.AppendLine("<tr>");
                firstColumn = true;
                Type propType = p.GetType();
                foreach (var propColumn in _actionPropertyColumn)             //property details
                {
                    var prop = propType.GetProperty(propColumn.Key);
                    object value = prop.GetValue(p, null);
                    if (value == null) value = "";

                    if (value is bool)
                        sb.AppendLine(bgColorCenter);
                    else
                        sb.AppendLine(bgColor);

                    if (firstColumn)
                    { // Hyperlink for first column
                        sb.AppendLine(value.ToString());
                        firstColumn = false;
                    }
                    else
                    {
                        if (value is bool)
                        { // Dispaly check mark for true boolean column.
                            if ((bool)value) sb.AppendLine("<img src=\"..\\image\\check.png\"/>");
                        }
                        else
                            sb.AppendLine(value.ToString());
                    }
                    sb.AppendLine("</td>");
                }

                Type extendedPropType = typeof(Data.Property);
                Data.Property propValue = null;
                if (isInput)
                    propValue = action.InputPropertyList.FirstOrDefault(pl => pl.Name == p.Name);
                else
                    propValue = action.OutputPropertyList.FirstOrDefault(pl => pl.Name == p.Name);
                
                foreach (var propColumn in _actionExtendedPropertyColumn)             //extended property details
                {
                    var prop = extendedPropType.GetProperty(propColumn.Key);
                    object value = prop.GetValue(propValue, null);
                    if (value == null) value = "";
                    if (value is bool)
                        sb.AppendLine(bgColorCenter);
                    else
                        sb.AppendLine(bgColor);

                    if (propColumn.Key == "Remarks")
                    { // Generate hyperlink for remarks as well as a remark page.
                        string remarks = (string)value;
                        if (!string.IsNullOrEmpty(remarks))
                        {
                            GeneratePropertyRemarks(action.Name, p.Name, remarks, string.Format(".Action.{0}.Remarks", inputOrOutput));
                            if (remarks.Length > 20) value = remarks.Substring(0, 20) + "...";
                            sb.AppendLine(string.Format("<a href=\"{0}.Action.{1}.Remarks.{2}.{3}.html\" title=\"{4}\">{4}</a>", _product, inputOrOutput, action.Name, p.Name, remarks, value.ToString()));
                        }
                    }
                    else
                    {
                        if (value is bool)
                        { // Display boolean property (true) with image.
                            if ((bool)value) sb.AppendLine("<img src=\"..\\image\\check.png\"/>");
                        }
                        else
                            sb.AppendLine(value.ToString());
                    }
                    sb.AppendLine("</td>");
                }
                sb.AppendLine("</tr>");
                i++;
                alternateColor = !alternateColor;
            }

            sb.AppendLine("</table>");

        }

        private void GenerateObsoleteActionPropertiesOverviewHtml(StringBuilder sb, bool isInput, Exact.Services.MetaModel.Action.Metadata meta, Data.Action action)
        {
            //var obsoleteProperties = entity.PropertyList.Where(pl => !(metadata.Interface.Properties.Select(p => p.Name).Contains(pl.Name))).OrderBy(pp => pp.Name);

            string title = string.Empty;
            Exact.Services.MetaModel.Action.Data.MetadataAction metadata = null;
            string inputOrOutput = string.Empty;
            Data.PropertyList propertyList = null;            

            // Setting up input and output configuration.
            if (isInput)
            {
                metadata = meta.RetrieveInputMetadata(action.Name);
                inputOrOutput = "input";
                propertyList = action.InputPropertyList;
            }
            else
            {
                metadata = meta.RetrieveOutputMetadata(action.Name);
                inputOrOutput = "output";
                propertyList = action.OutputPropertyList;
            }

            sb.AppendFormat("<h2>List of obsolete {0} property</h2>{1}", inputOrOutput, Environment.NewLine);
            var obsoleteProperty = propertyList.Where(pl => !(metadata.Properties.Select(p => p.Name).Contains(pl.Name)));
            if (metadata.Properties.Count == 0 || obsoleteProperty.Count()==0) sb.AppendFormat("<p>There is no obsolete {0} property for {1} action</p>{2}", inputOrOutput, action.Name, Environment.NewLine);            

            sb.AppendLine(tableConstantStart);

            int i = 0;
            bool alternateColor = false;
            bool isHeader = true;
            foreach (var p in obsoleteProperty.OrderBy(pl=>pl.Name))
            {
                if (isHeader)
                { // Generate header
                    sb.AppendLine("<tr>");
                    sb.AppendFormat(headerTextFormat, "Name", Environment.NewLine);
                    foreach (var prop in _actionExtendedPropertyColumn) //extended property header
                        sb.AppendFormat(headerTextFormat, prop.Value, Environment.NewLine);
                    sb.AppendLine("</tr>");
                }

                string bgColor = alternateColor ? "<td>" : alternateColorFormat;
                string bgColorCenter = alternateColor ? centerAlignedFormat : alternateColorCenterAlignedFormat;
                isHeader = false;
                sb.AppendLine("<tr>");
                sb.AppendLine(bgColor).Append(p.Name).AppendLine("</td>");

                Type extendedPropType = typeof(Data.Property);
                Data.Property propValue = null;
                propValue = propertyList.FirstOrDefault(pl => pl.Name == p.Name);

                foreach (var propColumn in _actionExtendedPropertyColumn)             //extended property details
                {
                    var prop = extendedPropType.GetProperty(propColumn.Key);
                    object value = prop.GetValue(propValue, null);
                    if (value == null) value = "";
                    if (value is bool)
                        sb.AppendLine(bgColorCenter);
                    else
                        sb.AppendLine(bgColor);

                    if (propColumn.Key == "Remarks")
                    { // Generate hyperlink for remarks as well as a remark page.
                        string remarks = (string)value;
                        if (!string.IsNullOrEmpty(remarks))
                        {
                            GeneratePropertyRemarks(action.Name, p.Name, remarks, string.Format(".Action.{0}.Remarks", inputOrOutput));
                            if (remarks.Length > 20) value = remarks.Substring(0, 20) + "...";
                            sb.AppendLine(string.Format("<a href=\"{0}.Action.{1}.Remarks.{2}.{3}.html\" title=\"{4}\">{4}</a>", _product, inputOrOutput, action.Name, p.Name, remarks, value.ToString()));
                        }
                    }
                    else
                    {
                        if (value is bool)
                        { // Display boolean property (true) with image.
                            if ((bool)value) sb.AppendLine("<img src=\"..\\image\\check.png\"/>");
                        }
                        else
                            sb.AppendLine(value.ToString());
                    }
                    sb.AppendLine("</td>");
                }
                sb.AppendLine("</tr>");
                i++;
                alternateColor = !alternateColor;
            }

            sb.AppendLine("</table>");

        }

        // Generate action properties
        private void GenerateActionPropertiesOverviewHtml()
        {
            mainProgress.Progress("Generating  Action's properties...");
            StringBuilder sbError = new StringBuilder();
            Exact.Services.MetaModel.Action.Metadata meta = new Exact.Services.MetaModel.Action.Metadata();
            foreach (var solutionArea in Helper.Header.SolutionArea)
            {
                if (solutionArea.ActionList != null)
                {
                    foreach (var action in solutionArea.ActionList)
                    {
                        try
                        {
                            StringBuilder sb = new StringBuilder();
                            sb.AppendLine("<html>");
                            sb.AppendLine("<body>");
                            sb.AppendLine("<h1>List of properties</h1>");

                            GenerateActionPropertiesOverviewHtml(sb, true, meta, action);
                            GenerateActionPropertiesOverviewHtml(sb, false, meta, action);
                            GenerateObsoleteActionPropertiesOverviewHtml(sb, true, meta, action);
                            GenerateObsoleteActionPropertiesOverviewHtml(sb, false, meta, action);

                            sb.AppendLine("</body>");
                            sb.AppendLine("</html>");
                            string file = string.Format("{0}{1}.Action.{2}.html", Helper.DocumentationHtmlPath, _product, action.Name);
                            File.WriteAllText(file, sb.ToString());
                        }
                        catch (Exception ex)
                        {
                            sbError.AppendLine(string.Format("{0}:{1} ({2})", solutionArea.Name, action.Name, ex.Message));
                            if (ex.InnerException != null) sbError.AppendLine(string.Format(" - {0}", ex.InnerException.Message));
                        }
                    }
                }
            }
            if (sbError.Length > 0) MessageBox.Show(sbError.ToString(), this.Title, MessageBoxButton.OK, MessageBoxImage.Warning);
        }

        private void GenerateActionProvidersOverviewHtml()
        {
            mainProgress.Progress("Generating Actions...");
            Type providerType = typeof(Data.ActionProvider);
            if (Helper.Header.ActionProvider != null)
            {
                foreach (var ap in Helper.Header.ActionProvider.OrderBy(e => e.Name))
                {
                    StringBuilder sb = new StringBuilder();
                    sb.AppendLine("<html>");
                    sb.AppendLine("<body>");
                    sb.AppendLine("<h1>Action provider details</h1>");

                    sb.AppendLine(tableConstantStart);
                    foreach (var epc in _entityProviderColumn)
                    {
                        sb.AppendLine("<tr>");
                        sb.AppendFormat(headerTextFormat, epc.Value, Environment.NewLine);
                        sb.AppendLine(string.Format("<td>{0}</td>", providerType.GetProperty(epc.Key).GetValue(ap, null)));
                        sb.AppendLine("</tr>");
                    }
                    sb.AppendLine("</table>");

                    sb.AppendLine("</body>");
                    sb.AppendLine("</html>");

                    string file = string.Format("{0}{1}.ActionProvider.{2}.html", Helper.DocumentationHtmlPath, _product, ap.Name);
                    File.WriteAllText(file, sb.ToString());
                }
            }
        }

        #endregion

    }
}
