﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Reflection;
using System.IO;
using System.Runtime.Serialization;

namespace EntityDocumentation
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private Data.Product _product = new Data.Product();
        public MainWindow()
        {
            InitializeComponent();
        }

        private bool IsProductPathValid()
        {
            //if (!string.IsNullOrEmpty(filePath.Text))
            //{
            //    MessageBox.Show("Please enter your product path.", "Warning");
            //    return false;
            //}

            bool valid = Helper.IsProductPathValid;
            if (!valid) MessageBox.Show(string.Format("Product path {0} is not a valid.{1}Please ensure a valid product path is entered.", Helper.ProductBinPath, Environment.NewLine), this.Title, MessageBoxButton.OK, MessageBoxImage.Error);
            return valid;
        }

        private void ClickMaintainEntity(object sender, RoutedEventArgs e)
        {
            if (IsProductPathValid())
            {
                Overview.Entity f = new Overview.Entity();
                f.ShowDialog();
            }
        }

        private void AddProduct()
        {
            //Only need to perform this once. filepath will be disabled the first time.
            if (filePath.IsEnabled)
            {
                bool isAddPath = true;
                foreach (Data.FilePath p in _product.Path)
                {
                    if (p.Path.ToLower() == _product.LastPath.ToLower())
                    {
                        isAddPath = false;
                        break;
                    }
                }

                if (isAddPath)
                {
                    _product.Path.Add(new Data.FilePath { Path = _product.LastPath });
                }
            }
        }

        private void AddPath()
        {
            Helper.ProductBinPath = _product.LastPath;
        }
        Assembly currentDomain_AssemblyResolve(object sender, ResolveEventArgs args)
        {
            //This handler is called only when the common language runtime tries to bind to the assembly and fails.

            AddProduct();
            AddPath();

            //Retrieve the list of referenced assemblies in an array of AssemblyName.
            Assembly MyAssembly, objExecutingAssemblies;
            string strTempAssmbPath = "";

            objExecutingAssemblies = Assembly.GetExecutingAssembly();
            AssemblyName[] arrReferencedAssmbNames = objExecutingAssemblies.GetReferencedAssemblies();

            //Loop through the array of referenced assembly names.
            foreach (AssemblyName strAssmbName in arrReferencedAssmbNames)
            {
                //Check for the assembly names that have raised the "AssemblyResolve" event.
                if (strAssmbName.FullName.Substring(0, strAssmbName.FullName.IndexOf(",")) == args.Name.Substring(0, args.Name.IndexOf(",")))
                {
                    //Build the path of the assembly from where it has to be loaded.
                    //The following line is probably the only line of code in this method you may need to modify:
                    strTempAssmbPath = _product.LastPath;
                    if (!strTempAssmbPath.EndsWith("\\")) strTempAssmbPath += "\\";
                    strTempAssmbPath += args.Name.Substring(0, args.Name.IndexOf(",")) + ".dll";
                    break;
                }

            }

            //This part here is to enable loading of DLL that is loaded dynamically 
            if (string.IsNullOrEmpty(strTempAssmbPath))
            {
                strTempAssmbPath = _product.LastPath;
                if (!strTempAssmbPath.EndsWith("\\")) strTempAssmbPath += "\\";

                if (args.Name.Substring(0, args.Name.IndexOf(",")) == "Exact.Synchronization.WinServiceHost")
                    strTempAssmbPath += args.Name.Substring(0, args.Name.IndexOf(",")) + ".exe";
                else
                    strTempAssmbPath += args.Name.Substring(0, args.Name.IndexOf(",")) + ".dll";

                if (!System.IO.File.Exists(strTempAssmbPath)) strTempAssmbPath = "";
            }

            //Load the assembly from the specified path.
            MyAssembly = Assembly.LoadFrom(strTempAssmbPath);

            filePath.IsEnabled = false;

            //Return the loaded assembly.
            return MyAssembly;
        }

        private void filePath_LostFocus(object sender, RoutedEventArgs e)
        {
            if (string.IsNullOrWhiteSpace(_product.LastPath)) return;
            if (_product.LastPath.EndsWith("\\")) _product.LastPath = _product.LastPath.Remove(_product.LastPath.LastIndexOf("\\"));
            AddPath();
            //if (Helper.Header.SolutionArea == null || Helper.Header.SolutionArea.Count ==0) 
            FilePathSelected();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            if (!Helper.ReadSetting(ref _product, string.Format("{0}\\EntityDocumentation.xml", AppDomain.CurrentDomain.BaseDirectory)))
            {
                this.Close();
                return;
            }
            if (_product.LastPath == null) _product.LastPath = string.Empty;
            if (_product.Path == null) _product.Path = new List<Data.FilePath>();
            filePath.ItemsSource = _product.Path;
            if (System.IO.File.Exists(string.Format("{0}\\Exact.Utilities.DLL", AppDomain.CurrentDomain.BaseDirectory)))
            {
                _product.LastPath = AppDomain.CurrentDomain.BaseDirectory.Remove(AppDomain.CurrentDomain.BaseDirectory.LastIndexOf("\\"));
                Helper.ProductBinPath = _product.LastPath;
                filePath.IsEnabled = false;
            }
            if (string.IsNullOrEmpty(Helper.ProductBinPath)) Helper.ProductBinPath = _product.LastPath;
            Helper.Header.SolutionArea = new Data.SolutionAreaList();
            FilePathSelected();
            main.DataContext = _product;            
        }

        private void Window_Closed(object sender, EventArgs e)
        {
            Helper.WriteSetting(_product, string.Format("{0}\\EntityDocumentation.xml", AppDomain.CurrentDomain.BaseDirectory));
        }

        private void Window_Initialized(object sender, EventArgs e)
        {
            AppDomain currentDomain = AppDomain.CurrentDomain;
            currentDomain.AssemblyResolve += new ResolveEventHandler(currentDomain_AssemblyResolve);
        }

        private void ClickImportFromServiceLocator(object sender, RoutedEventArgs e)
        {
            //if (!string.IsNullOrEmpty(filePath.Text))
            //{
            if (IsProductPathValid())
            {
                Overview.ImportFromServiceLocator f = new Overview.ImportFromServiceLocator();
                f.ShowDialog();
            }
            //}
            //else
            //  MessageBox.Show("Please enter your product path.","Warning");
        }

        private void ClickGenerateHtml(object sender, RoutedEventArgs e)
        {
            if (IsProductPathValid())
            {
                Html.Entity f = new Html.Entity(false);
                f.ShowDialog();
            }
        }

        private void ClickViewHtml(object sender, RoutedEventArgs e)
        {
            if (IsProductPathValid())
            {
                Html.Entity f = new Html.Entity(true);
                f.ShowDialog();
            }
        }

        private void filePathSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            _product.LastPath = (string)filePath.SelectedValue;
            AddPath();
            FilePathSelected();
        }
        private void FilePathSelected()
        {
            string productPrefix = Helper.ProductPrefix;
            if (string.IsNullOrEmpty(productPrefix)) return;
            if (productPrefix == Helper.Header.Prefix) return;
            string file = string.Format("{0}\\xmd\\Documentation.{1}.xml", Helper.DocumentationDomainPath, productPrefix);
            if (!Helper.ReadSetting<Data.DocumentHeader>(ref Helper.Header, file)) this.Close();
            Helper.Header.Prefix = productPrefix;
            if (Helper.Header.SolutionArea == null) Helper.Header.SolutionArea = new Data.SolutionAreaList();
            Helper.SetProductRelease();
        }


        private void ClickMaintainEntityProvier(object sender, RoutedEventArgs e)
        {
            if (IsProductPathValid())
            {
                Overview.EntityProvider f = new Overview.EntityProvider();
                f.ShowDialog();
            }
        }

        private void ClickMaintainEntitiesProvier(object sender, RoutedEventArgs e)
        {
            if (IsProductPathValid())
            {
                Overview.EntitiesProvider f = new Overview.EntitiesProvider();
                f.ShowDialog();
            }
        }

        private void ClickSave(object sender, RoutedEventArgs e)
        {
            string productPrefix = Helper.ProductPrefix;
            if (string.IsNullOrEmpty(productPrefix))
            {
                MessageBox.Show("Unregconized product", this.Title, MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }
            foreach (var solution in Helper.Header.SolutionArea) solution.BuildSaveVersion();
            string file = string.Format("{0}\\xmd\\Documentation.{1}.xml", Helper.DocumentationDomainPath, productPrefix);
            Helper.WriteSetting<Data.DocumentHeader>(Helper.Header, file);
        }

        private void ClickMaintainAction(object sender, RoutedEventArgs e)
        {
            if (IsProductPathValid())
            {
                Overview.Action f = new Overview.Action();
                f.ShowDialog();
            }
        }

        private void ClickMaintainActionProvider(object sender, RoutedEventArgs e)
        {
            if (IsProductPathValid())
            {
                Overview.ActionProvider f = new Overview.ActionProvider();
                f.ShowDialog();
            }
        }
    }
}
